 // Glen Popiel, KW5GP

// Based on OpenQRP Blog Iambic Morse Code Keyer Sketch by Steven T. Elliott, K1EL - Used with Permission

/////////////////////////////////////////////////////////////////////////////////////////
//
//  Iambic Morse Code Keyer Sketch
//  Copyright (c) 2009 Steven T. Elliott
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details:
//
//  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA  02111-1307  USA
//
/////////////////////////////////////////////////////////////////////////////////////////

//#define debug  // Enables Debug mode

#include <Adafruit_GFX.h> // The Adafruit Graphics Library
#include <Adafruit_ILI9341.h> // The ILI9341 TFT Display Library
#include <SPI.h>

#define BAUD_RATE 115200
#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_DC     9  // Assign the TFT DC to pin 9

#define font_size 3  // Ther display font size

#define tft_cyan ILI9341_CYAN // Simplifies the color definitions for the TFT
#define tft_black ILI9341_BLACK

#define ST_Pin 4      // Sidetone Output Pin on Pin 4
#define LP_in 7       // Left Paddle Input on Pin 7
#define RP_in 5       // Right Paddle Input on Pin 5
#define led_Pin A1    // LED on Pin A1
#define Mode_A_Pin  3 // Mode Select Switch Side A
#define Mode_B_Pin  2 // Mode Select Switch Side B
#define key_Pin 6     // Transmitter Relay Key Pin
#define Speed_Pin A0   // Speed Control Pot on A0
#define ST_Freq 600   // Set the Sidetone Frequency to 600 Hz

#define no_beep 8    // Beep Disable Switch Input

int key_speed;        // variable for keying speed
int read_speed;       // variable for speed pot
int key_mode;         // variable for keying mode
int last_mode = -1;        // variable to detect keying mode change
boolean beep_enable = true;      // variable to select beep on/off
boolean last_beep = false;
boolean data_changed = false; // used to indicate if any status information has changed
int background = tft_black; // Set the TFT background and text color
int text_color = tft_cyan;

unsigned long ditTime;  // Number of milliseconds per dit
char keyerControl;
char keyerState;
String mode_text = " ";  // LCD text variable

static long ktimer;
int debounce;

/////////////////////////////////////////////////////////////////////////////////////////
//  keyerControl bit definitions
//
#define     DIT_L      0x01     // Dit latch
#define     DAH_L      0x02     // Dah latch
#define     DIT_PROC   0x04     // Dit is being processed
#define     PDLSWAP    0x08     // 0 for normal, 1 for swap
#define     IAMBICB    0x10     // 0 for Iambic A, 1 for Iambic B
#define     ULTIMATIC  0x20     // 1 for ultimatic
#define     STRAIGHT   0x80     // 1 for straight key mode 
////////////////////////////////////////////////////////////////////////////////////////
//
//  State Machine Defines

enum KSTYPE {
  IDLE, CHK_DIT, CHK_DAH, KEYED_PREP, KEYED, INTER_ELEMENT
};

// Use hardware SPI (on Uno/Nano, #13, #12, #11) and the above for CS/DC
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);  // Create the TFT instance

void setup()
{
  #ifdef debug
  Serial.begin(BAUD_RATE); // initialize serial communication on the USB port
  #endif

  // Setup outputs
  pinMode(led_Pin, OUTPUT);         // sets the LED digital pin as output
  pinMode(LP_in, INPUT_PULLUP);            // sets Left Paddle digital pin as input
  pinMode(RP_in, INPUT_PULLUP);            // sets Right Paddle digital pin as input
  pinMode(ST_Pin, OUTPUT);          // Sets the Sidetone digital pin as output
  pinMode(key_Pin, OUTPUT);         // Sets the Keying Relay pin as output
  pinMode(Mode_A_Pin, INPUT_PULLUP);       // sets Iambic Mode Switch Side A as input
  pinMode(Mode_B_Pin, INPUT_PULLUP);       // sets Iambic Mode Switch Side B as input
  digitalWrite(led_Pin, LOW);       // turn the LED off
  pinMode(no_beep, INPUT_PULLUP);          // Sets Beep Disable digital pin as input
 
  keyerState = IDLE;
  keyerControl = 0;

  key_speed = map(analogRead(Speed_Pin), 20, 1000, 5, 35); // Read the potentiometer to determine code speed
  loadWPM(key_speed);   // Set the keying speed

  start_tft();  // Start the TFT display

  delay(3000);

  update_tft();

}  // End Setup Loop

void loop()
{
  // Read and set speed
  read_speed = map(analogRead(Speed_Pin), 20, 1000, 5, 35); // Read the potentiometer to determine code speed
 if (read_speed >35)
 {
  read_speed = 35;
 }
 if (read_speed <5)
 {
  read_speed = 5;
 }
  if (key_speed != read_speed)  // If the Speed Pot has changed, update the speed and LCD
  {
    key_speed = read_speed;
    loadWPM(key_speed);
    data_changed = true;
  }

/*
  Read the Mode Switch and set mode
  
  Key Mode 0 = Iambic Mode A
  Key Mode 1 = Iambic Mode B
  Key Mode 2 = Straight Key
*/

  if (digitalRead(Mode_A_Pin) == LOW)   // Set Iambic Mode A
  {
    key_mode = 0;
    mode_text = "Iambic A";
  }
  if (digitalRead(Mode_B_Pin) == LOW)  // Set Iambic Mode B
  {
    key_mode = 1;
    mode_text = "Iambic B";
  }
  if (digitalRead(Mode_A_Pin) == HIGH && digitalRead(Mode_B_Pin) == HIGH)  // Set Straight Key
  {
    key_mode = 2;
    mode_text = "Straight";
  }
  if (key_mode != last_mode)
  {
    last_mode = key_mode;
    data_changed = true;
  }

beep_enable = digitalRead(no_beep);
if (beep_enable != last_beep) // Switch has changed
{
  data_changed = true;
  last_beep = beep_enable;
  
}

  if (data_changed)
  {
    update_tft();
    data_changed = false;
  }

  if (key_mode == 2) // Straight Key
  {
    // Straight Key Mode
    if ((digitalRead(LP_in) == LOW) || (digitalRead(RP_in) == LOW))
    {
      // Key from either paddle
      digitalWrite(led_Pin, HIGH);
      digitalWrite(key_Pin, HIGH);
      if (beep_enable)
      {
        tone(ST_Pin, 600);
      }
    }
    else {
      digitalWrite(led_Pin, LOW);
      digitalWrite(key_Pin, LOW);
      noTone(ST_Pin);
    }
  }
  else
  {
    // Basic Iambic Keyer
    // keyerControl contains processing flags and keyer mode bits
    // Supports Iambic A and B
    // State machine based, uses calls to millis() for timing.
    switch (keyerState)
    {
      case IDLE:      // Wait for direct or latched paddle press
        if ((digitalRead(LP_in) == LOW) || (digitalRead(RP_in) == LOW) || (keyerControl & 0x03))
        {
          update_PaddleLatch();
          keyerState = CHK_DIT;
        }
        break;

      case CHK_DIT:      // See if the dit paddle was pressed
        if (keyerControl & DIT_L)
        {
          keyerControl |= DIT_PROC;
          ktimer = ditTime;
          keyerState = KEYED_PREP;
        }
        else {
          keyerState = CHK_DAH;
        }
        break;

      case CHK_DAH:      // See if dah paddle was pressed
        if (keyerControl & DAH_L)
        {
          ktimer = ditTime * 3;
          keyerState = KEYED_PREP;
        }
        else {
          keyerState = IDLE;
        }
        break;

      case KEYED_PREP:      // Assert key down, start timing, state shared for dit or dah
        digitalWrite(led_Pin, HIGH);         // Turn the LED on
        if (beep_enable)
        {
          tone(ST_Pin, ST_Freq);               // Turn the Sidetone on
        }
        digitalWrite(key_Pin, HIGH);         // Key the TX Relay
        ktimer += millis();                  // set ktimer to interval end time
        keyerControl &= ~(DIT_L + DAH_L);    // clear both paddle latch bits
        keyerState = KEYED;                  // next state
        break;

      case KEYED:      // Wait for timer to expire
        if (millis() > ktimer)  // are we at end of key down ?
        {
          digitalWrite(led_Pin, LOW);      // Turn the LED off
          noTone(ST_Pin);                  // Turn the Sidetone off
          digitalWrite(key_Pin, LOW);      // Turn the TX Relay off
          ktimer = millis() + ditTime;     // inter-element time
          keyerState = INTER_ELEMENT;      // next state
        }
        else if (key_mode == 1) // Check to see if we're in Iambic B Mode
        {
          update_PaddleLatch();           // early paddle latch in Iambic B mode
        }
        break;

      case INTER_ELEMENT:      // Insert time between dits/dahs
        update_PaddleLatch();               // latch paddle state
        if (millis() > ktimer)  // are we at end of inter-space ?
        {
          if (keyerControl & DIT_PROC)      // was it a dit or dah ?
          {
            keyerControl &= ~(DIT_L + DIT_PROC);   // clear two bits
            keyerState = CHK_DAH;                  // dit done, check for dah
          }
          else {
            keyerControl &= ~(DAH_L);              // clear dah latch
            keyerState = IDLE;                     // go idle
          }
        }
        break;
    }
  }
}  // End Main Loop

///////////////////////////////////////////////////////////////////////////////
//
//    Latch dit and/or dah press
//
//    Called by keyer routine
//
///////////////////////////////////////////////////////////////////////////////

void update_PaddleLatch()
{
  if (digitalRead(RP_in) == LOW) {
    keyerControl |= DIT_L;
  }
  if (digitalRead(LP_in) == LOW) {
    keyerControl |= DAH_L;
  }
}

///////////////////////////////////////////////////////////////////////////////
//
//    Calculate new time constants based on wpm value
//
///////////////////////////////////////////////////////////////////////////////

void loadWPM (int wpm)
{
  ditTime = 1200 / wpm;
}

void start_tft()  // Starts the TFT display and shows the startup screen
{
  tft.begin();  // Start the TFT display

#ifdef debug1
  Serial.println("TFT Start");  // Verifies that the TFT is operational
  Serial.print("Display Pwr Mode: 0x");
  Serial.println(tft.readcommand8(ILI9341_RDMODE), HEX);
#endif

  clear_display();  // Clear the TFT display
  tft.setTextColor(text_color, background);  // Set the display text to black. Green background color required to "clear" previous text
  tft.setRotation(3); // horizontal display // change rotation as needed/desired
  tft.setTextSize(font_size); // large font

  tft.setCursor(110, 30);  //Set the Cursor and display the startup screen
  tft.print("KW5GP");
  tft.setCursor(100, 90);
  tft.print("Iambic");
  tft.setCursor(110, 150);
  tft.print("Keyer");

  delay(5000);  //Wait 5 seconds then clear the startup message
  clear_display();  // Clear the TFT display
}

void clear_display()  // Function to clear the TFT display
{
  tft.fillScreen(background); // Clear the display using current background color
}

void update_tft() // displays keyer settings on the TFT
{
  tft.setCursor(70, 5);
  tft.print("Keyer Ready");
  tft.setCursor(10, 65);
  tft.print("CW Speed: ");
  tft.print(key_speed);
  tft.print(" ");
  tft.setCursor(10, 125);
  tft.print("Mode: ");
  tft.print(mode_text);
  tft.print("   ");
  tft.setCursor(10,185);
  tft.print("Tone: ");
  if (beep_enable)
  {
    tft.print("Enabled ");
  }
  else
  {
    tft.print("Disabled");
  }
}
